--Core of addon
MorthalinRaidTools = LibStub("AceAddon-3.0"):NewAddon("Morthalin Raid Tools", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0", "AceSerializer-3.0")
local MorthalinRaidTools	= LibStub("AceAddon-3.0"):GetAddon("Morthalin Raid Tools")
local AceGUI = LibStub("AceGUI-3.0")
local LibDeflate = LibStub:GetLibrary("LibDeflate")

--COMMAND options
local options = {
	name = 'MorthalinRaidTools',
	type = 'group',
	args = {
		bench = {
			type = 'input',
			name = "Add member",
			desc = "Add guild member on bench /mrt bench playername",
			usage = "player name",
			get = false,
			set = function(info, playerName) MorthalinRaidTools:AddOnBench(playerName, playerName); end
		},
		rm = {
			type = 'input',
			name = "Remove member",
			desc = "Remove member from bench /mrt rm playername",
			usage = "player name",
			get = false,
			set = function(info, playerName) MorthalinRaidTools:RemoveFromBench(playerName); end
		},
		list = {
			type = 'execute',
			name = "Show players",
			desc = "Show acutal list of players on bench",
			func = function() MorthalinRaidTools:GetList(); end
		},
		history = {
			type = 'execute',
			name = "Show history",
			desc = "Show history list",
			func = function() MorthalinRaidTools:ShowHistory(); end
		},
		reset = {
			type = 'execute',
			name = "Empty bench",
			desc = "Clear list of players on bench",
			func = function() MorthalinRaidTools:ResetList(); end
		},
		resetHistory = {
			type = 'execute',
			name = "Empty history",
			desc = "Clear list of kills",
			func = function() MorthalinRaidTools:ResetHistory(); end
		},
		resetGamble = {
			type = 'execute',
			name = "Reset gamble statistics",
			desc = "Clear gamble statistics",
			func = function() MorthalinRaidTools:ClearGambleStatistics(); end
		},
		sendBuffs = {
			type = 'input',
			name = "Send buff settings to player",
			desc = "Send buff settings to player /mrt sendBuffs playername",
			usage = "player name",
			get = false,
			set = function(info, playerName) MorthalinRaidTools:SendBuffSettings(playerName); end
		},
	}
}

local settings = {
	name = 'Morthalin Raid Tools Settings',
	type = 'group',
	args = {
		broadcastRequestHeader = {
			type = "header",
			name = "Broadcast requests",
			order = 1,
		},
		broadcastRequestBench = {
			order = 2,
			type = "execute",
			name = "Request bench",
			desc = "Request for update players on bench.",
			func = function() MorthalinRaidTools:AskForBenchBroadcast(); end,
		},
		broadcastRequest7History = {
			order = 3,
			type = "execute",
			name = "Request history 7D",
			desc = "Request for broadcast all kills of last 7 days.",
			func = function() MorthalinRaidTools:AskForHistory7Broadcast(); end,
		},
		broadcastHeader = {
			type = "header",
			name = "Broadcast options",
			order = 4,
		},
		broadcast = {
			order = 5,
			type = "toggle",
			name = "Allow broadcasts",
			desc = "Allow broadcasts from guild.",
			get = function() return MorthalinRaidTools.db.global.allowBroadcast; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowBroadcast = v; end,
		},
		timeStampFormat = {
			type = "select",
			name = "Allow broadcast from rank",
			desc = "Set rank which sender need to broadcast to you.",
			values = function()
				local ranks = {}
				for i = 1, GuildControlGetNumRanks() do
					table.insert(ranks, GuildControlGetRankName(i));
				end
				return ranks;
			end,
			order = 6,
			get = function() return MorthalinRaidTools.db.global.broadcastIncomingRank; end,
			set = function(info, v) MorthalinRaidTools.db.global.broadcastIncomingRank = v; end,
		},
		broadcastAll = {
			order = 7,
			type = "execute",
			name = "Broadcast history",
			desc = "Broadcast full history of kills.",
			func = function() MorthalinRaidTools:BroadcastHistory(); end,
		},
		broadcastRemove = {
			order = 8,
			type = "toggle",
			name = "Allow remove",
			desc = "Allow broadcasts for remove players of instances from history.",
			get = function() return MorthalinRaidTools.db.global.allowBroadcastRemove; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowBroadcastRemove = v; end,
		},
		broadcastOnlyGuild = {
			order = 9,
			type = "toggle",
			name = "Only guild raids",
			desc = "Allow broadcasts only raids where was at least 50% of guild members.",
			get = function() return MorthalinRaidTools.db.global.allowOnlyGuildBroadcast; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowOnlyGuildBroadcast = v; end,
		},
		mainBroadcaster = {
			order = 10,
			type = "toggle",
			name = "Main broadcaster",
			desc = "Set self as broadcaster for required bench and history broadcasts.",
			get = function() return MorthalinRaidTools.db.global.mainBroadcaster; end,
			set = function(info, v) MorthalinRaidTools.db.global.mainBroadcaster = v; end,
		},
		historyHeader = {
			type = "header",
			name = "History options",
			order = 11,
		},
		historyTooltip = {
			order = 12,
			type = "toggle",
			name = "AutoHide Tooltip",
			desc = "Automatically hide tooltip on history view.",
			get = function() return MorthalinRaidTools.db.global.autoHideTooltip; end,
			set = function(info, v) MorthalinRaidTools.db.global.autoHideTooltip = v; end,
		},
		historyOnlyGuild = {
			order = 13,
			type = "toggle",
			name = "Only guild raids",
			desc = "Allow save only raids where was at least 50% of guild members.",
			get = function() return MorthalinRaidTools.db.global.allowOnlyGuildKills; end,
			set = function(info, v) MorthalinRaidTools.db.global.allowOnlyGuildKills = v; end,
		},
		historyBuffsSettings = {
			order = 14,
			type = "execute",
			name = "Buffs Settings",
			desc = "Settings of visible buffs in history",
			func = function() MorthalinRaidTools:ConstructBuffsSettings(); end,
		},
		benchHeader = {
			type = "header",
			name = "Bench options",
			order = 15,
		},
		autoBench = {
			order = 16,
			type = "toggle",
			name = "Auto bench invites",
			desc = "This option will automatically allow bench invites after raid formed by whisper !bench or !bench 'nickname' if it was disabled.",
			get = function() return MorthalinRaidTools.db.global.checkBenchAuto; end,
			set = function(info, v) MorthalinRaidTools.db.global.checkBenchAuto = v; end,
		},
		autoBenchEmpty= {
			order = 17,
			type = "toggle",
			name = "Ask for empty bench",
			desc = "This option will automatically ask for empty bench after raid leaved.",
			get = function() return MorthalinRaidTools.db.global.autoBenchEmpty; end,
			set = function(info, v) MorthalinRaidTools.db.global.autoBenchEmpty = v; end,
		},
		statisticsHeader = {
			type = "header",
			name = "Statistics options",
			order = 18,
		},
		mainRaids = {
			type = "multiselect",
			name = "Main raids",
			desc = "Set main raids to count attendance on.",
			values = function()
				local uniqueZones = {};
				local zones = {};
				for i = 1, #MorthalinRaidTools.db.global.history do
					if(uniqueZones[MorthalinRaidTools.db.global.history[i].zone] == nil) then
						uniqueZones[MorthalinRaidTools.db.global.history[i].zone] = {zone = MorthalinRaidTools.db.global.history[i].zone};
					end
				end
				
				for index, value in pairs(uniqueZones) do
					table.insert(zones, index);
				end
				local function sortZones(a, b)
					return a < b;
				end
				table.sort(zones, sortZones);
				
				if(#MorthalinRaidTools.db.global.mainRaids < #zones) then
					for i = #MorthalinRaidTools.db.global.mainRaids + 1, #zones do
						table.insert(MorthalinRaidTools.db.global.mainRaids, {zone = zones[i], value = true});
					end
				end
				return zones;
			end,
			order = 19,
			get = function(info, index) return MorthalinRaidTools.db.global.mainRaids[index].value; end,
			set = function(info, index, value) MorthalinRaidTools.db.global.mainRaids[index].value = value; end,
		},
		gambleHeader = {
			type = "header",
			name = "Gamble options",
			order = 20,
		},
		autoGamble = {
			order = 21,
			type = "toggle",
			name = "Auto game timers",
			desc = "This option will automatically process game with set timers.",
			get = function() return MorthalinRaidTools.db.global.gambleAuto; end,
			set = function(info, v) MorthalinRaidTools.db.global.gambleAuto = v; end,
		},
		autoJoinGamble = {
			order = 22,
			type = "toggle",
			name = "Auto join",
			desc = "This option will automatically join and roll.",
			get = function() return MorthalinRaidTools.db.global.gambleJoin; end,
			set = function(info, v) MorthalinRaidTools.db.global.gambleJoin = v; end,
		},
		inviteTimer = {
			order = 23,
			type = "input",
			name = "Invite time",
			desc = "Time interval before last call.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.gambleInviteTime); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.gambleInviteTime = tonumber(v); end end,
		},
		lastCallTimer = {
			order = 24,
			type = "input",
			name = "Last call time",
			desc = "Time interval before start rolling.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.gambleLastCallTime); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.gambleLastCallTime = tonumber(v); end end,
		},
		ReminderTimer = {
			order = 25,
			type = "input",
			name = "Reminder",
			desc = "Time interval to remind players not rolled.",
			width = "half",
			get = function() return tostring(MorthalinRaidTools.db.global.gambleRollReminder); end,
			set = function(info, v) if (tonumber(v) ~= nil) then MorthalinRaidTools.db.global.gambleRollReminder = tonumber(v); end end,
		},
	}
}

--SCHEME FOR DB
local defaults = {
	global = {
		bench = {
			players = {},
		},
		version = 0,
		acceptBench = false,
		allowBroadcast = true,
		allowBroadcastRemove = false,
		allowOnlyGuildBroadcast = true,
		allowOnlyGuildKills = true,
		mainBroadcaster = false,
		broadcastIncomingRank = 1,
		mainRaids = {},
		autoHideTooltip = true,
		checkBenchAuto = true,
		autoBenchEmpty = true,
		minimap = {hide = false},
		history = {},
		members = {},
		showBuffs = true,
		showBuffsAll = true,
		buffs = {},
		itemCounter = {},
		gambleStatistics = {},
		gambleAuto = false,
		gambleJoin = false,
		gambleInviteTime = 10,
		gambleLastCallTime = 3,
		gambleRollReminder = 5,
		encounterList = {
			["Lucifron"] = {
				id = 663,
				name = "Lucifron",
				instance = "Molten Core",
			},
			["Magmadar"] = {
				id = 664,
				name = "Magmadar",
				instance = "Molten Core",
			},
			["Gehennas"] = {
				id = 665,
				name = "Gehennas",
				instance = "Molten Core",
			},
			["Garr"] = {
				id = 666,
				name = "Garr",
				instance = "Molten Core",
			},
			["Shazzrah"] = {
				id = 667,
				name = "Shazzrah",
				instance = "Molten Core",
			},
			["Baron Geddon"] = {
				id = 668,
				name = "Baron Geddon",
				instance = "Molten Core",
			},
			["Sulfuron Harbinger"] = {
				id = 669,
				name = "Sulfuron Harbinger",
				instance = "Molten Core",
			},
			["Golemagg the Incinerator"] = {
				id = 670,
				name = "Golemagg the Incinerator",
				instance = "Molten Core",
			},
			["Majordomo Executus"] = {
				id = 671,
				name = "Majordomo Executus",
				instance = "Molten Core",
			},
			["Ragnaros"] = {
				id = 672,
				name = "Ragnaros",
				instance = "Molten Core",
			},
			["Razorgore the Untamed"] = {
				id = 610,
				name = "Razorgore the Untamed",
				instance = "Blackwing Lair",
			},
			["Vaelastrasz the Corrupt"] = {
				id = 611,
				name = "Vaelastrasz the Corrupt",
				instance = "Blackwing Lair",
			},
			["Broodlord Lashlayer"] = {
				id = 612,
				name = "Broodlord Lashlayer",
				instance = "Blackwing Lair",
			},
			["Firemaw"] = {
				id = 613,
				name = "Firemaw",
				instance = "Blackwing Lair",
			},
			["Ebonroc"] = {
				id = 614,
				name = "Ebonroc",
				instance = "Blackwing Lair",
			},
			["Flamegor"] = {
				id = 615,
				name = "Flamegor",
				instance = "Blackwing Lair",
			},
			["Chromaggus"] = {
				id = 616,
				name = "Chromaggus",
				instance = "Blackwing Lair",
			},
			["Nefarian"] = {
				id = 617,
				name = "Nefarian",
				instance = "Blackwing Lair",
			},
			["Onyxia"] = {
				id = 1084,
				name = "Onyxia",
				instance = "Onyxia's Lair",
			},
			["High Priest Venoxis"] = {
				id = 784,
				name = "High Priest Venoxis",
				instance = "Zul'Gurub",
			},
			["High Priestess Jeklik"] = {
				id = 785,
				name = "High Priestess Jeklik",
				instance = "Zul'Gurub",
			},
			["High Priestess Mar'li"] = {
				id = 786,
				name = "High Priestess Mar'li",
				instance = "Zul'Gurub",
			},
			["Bloodlord Mandokir"] = {
				id = 787,
				name = "Bloodlord Mandokir",
				instance = "Zul'Gurub",
			},
			["Edge of Madness"] = {
				id = 788,
				name = "Edge of Madness",
				instance = "Zul'Gurub",
			},
			["High Priest Thekal"] = {
				id = 789,
				name = "High Priest Thekal",
				instance = "Zul'Gurub",
			},
			["Gahz'ranka"] = {
				id = 790,
				name = "Gahz'ranka",
				instance = "Zul'Gurub",
			},
			["High Priestess Arlokk"] = {
				id = 791,
				name = "High Priestess Arlokk",
				instance = "Zul'Gurub",
			},
			["Jin'do the Hexxer"] = {
				id = 792,
				name = "Jin'do the Hexxer",
				instance = "Zul'Gurub",
			},
			["Hakkar"] = {
				id = 793,
				name = "Hakkar",
				instance = "Zul'Gurub",
			},
			["The Prophet Skeram"] = {
				id = 709,
				name = "The Prophet Skeram",
				instance = "Ahn'Qiraj Temple",
			},
			["Silithid Royalty"] = {
				id = 710,
				name = "Silithid Royalty",
				instance = "Ahn'Qiraj Temple",
			},
			["Battleguard Sartura"] = {
				id = 711,
				name = "Battleguard Sartura",
				instance = "Ahn'Qiraj Temple",
			},
			["Fankriss the Unyielding"] = {
				id = 712,
				name = "Fankriss the Unyielding",
				instance = "Ahn'Qiraj Temple",
			},
			["Viscidus"] = {
				id = 713,
				name = "Viscidus",
				instance = "Ahn'Qiraj Temple",
			},
			["Princess Huhuran"] = {
				id = 714,
				name = "Princess Huhuran",
				instance = "Ahn'Qiraj Temple",
			},
			["Twin Emperors"] = {
				id = 715,
				name = "Twin Emperors",
				instance = "Ahn'Qiraj Temple",
			},
			["Ouro"] = {
				id = 716,
				name = "Ouro",
				instance = "Ahn'Qiraj Temple",
			},
			["C'thun"] = {
				id = 717,
				name = "C'thun",
				instance = "Ahn'Qiraj Temple",
			},
			["Kurinnaxx"] = {
				id = 718,
				name = "Kurinnaxx",
				instance = "Ruins of Ahn'Qiraj",
			},
			["General Rajaxx"] = {
				id = 719,
				name = "General Rajaxx",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Moam"] = {
				id = 720,
				name = "Moam",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Buru the Gorger"] = {
				id = 721,
				name = "Buru the Gorger",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Ayamiss the Hunter"] = {
				id = 722,
				name = "Ayamiss the Hunter",
				instance = "Ruins of Ahn'Qiraj",
			},
			["Ossirian the Unscarred"] = {
				id = 723,
				name = "Ossirian the Unscarred",
				instance = "Ruins of Ahn'Qiraj",
			}
		},
		instances = {
			["Molten Core"] = "MC",
			["Blackwing Lair"] = "BWL",
			["Onyxia's Lair"] = "Onyx",
			["Zul'Gurub"] = "ZG",
			["Ahn'Qiraj Temple"] = "AQ40",
			["Ruins of Ahn'Qiraj"] = "AQ20"
		},
		resets = {
			["ZG"] = 1587020400,
			["Onyx"] = 1586329200,
			["MC"] = 1586329200,
		}
	},
	char = {
		minimap = {hide = false},
		raid = IsInRaid(),
	}
}

--EVENTS
MorthalinRaidTools:RegisterEvent("BOSS_KILL", function(...) MorthalinRaidTools:BossKilled(...) end);

--Init function
function MorthalinRaidTools:OnInitialize()
	MorthalinRaidTools.db = LibStub("AceDB-3.0"):New("MorthalinRaidToolsDB", defaults);
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MorthalinRaidTools", options, {"morthalinRaidTools", "mrt"});
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MorthalinRaidTools-Settings", settings);
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MorthalinRaidTools-Settings", "Morthalin Raid Tools");
	MorthalinRaidTools.debug = false;
	if(MorthalinRaidTools.db.global.acceptBench) then
		MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
		MorthalinRaidTools:RegisterEvent("CHAT_MSG_RAID_LEADER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_GUILD", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	    MorthalinRaidTools:RegisterEvent("CHAT_MSG_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
	end
	MorthalinRaidTools:RegisterEvent("GROUP_ROSTER_UPDATE", function(...) MorthalinRaidTools:PartyChanged(...) end);
	MorthalinRaidTools:RegisterEvent("ENCOUNTER_START", function(...) MorthalinRaidTools:BossPulled(...) end);
	--MorthalinRaidTools:RegisterEvent("CHAT_MSG_LOOT", function(...) MorthalinRaidTools:LootMsg(...) end);
	MorthalinRaidTools:RegisterComm("MRTAddBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTRemoveBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTFullBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTEmptyBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTKill", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTHistory", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTAddHistPlayer", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTRemoveKill", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTAskForBench", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTAskForHist7", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:RegisterComm("MRTBuffSettings", function(...) MorthalinRaidTools:OnCommReceived(...) end);
	MorthalinRaidTools:CheckVersion();
	MorthalinRaidTools:ConstructUI();
	MorthalinRaidTools:ConstructMiniMapIcon();
end

-- Callback function for chat invites to raid
-- =========================
function MorthalinRaidTools:ChatChannelCallback(...)
	local message = select(2, ...)
	local sender = select(3, ...)
	message = message:gsub("%s+", "") -- trim whitespace
	sender = Ambiguate(sender, "short")
	if (message == "!bench") then
		MorthalinRaidTools:AddOnBench(sender, sender);
	elseif (string.sub(message, 1, 6) == "!bench") then
		MorthalinRaidTools:AddOnBench(string.sub(message, 7, -1), sender);
	elseif (message == "1" and MorthalinRaidTools:AcceptInvGamble()) then
		MorthalinRaidTools:AddPlayerGamble(sender);
	end
end

-- Callback function for boss pull
-- =========================
function MorthalinRaidTools:BossPulled(...)
	local event = select(1, ...);
	local encounterID = select(2, ...);
	local encounterName = select(3, ...);
	MorthalinRaidTools:ChechBuffs();
	MorthalinRaidTools:Print("Encounter started")
end

-- Callback function for boss kill
-- =========================
function MorthalinRaidTools:BossKilled(...)
	local event = select(1, ...);
	local encounterID = select(2, ...);
	local encounterName = select(3, ...);
	MorthalinRaidTools:AddBossKill(encounterID, encounterName);
end

-- Callback function for after invited
-- =========================
function MorthalinRaidTools:PartyChanged(...)
	if(self.db.char.raid ~= IsInRaid() and IsInRaid()) then
		self:OnRaidCreated();
	elseif(GetRaidRosterInfo(1) == null and self.db.char.raid) then
		self:OnRaidLeave();
	else
		self:CheckBenchInRaid();
	end
	
	self.db.char.raid = IsInRaid();
end

function MorthalinRaidTools:LootMsg(...)
	local msg = select(2, ...);
	local player = select(6, ...);
	if(string.sub(msg, 1, 17) == "You receive loot:") then
		if(MorthalinRaidTools.db.global.itemCounter[player] == nil) then
			MorthalinRaidTools.db.global.itemCounter[player] = {item = {}};
		end
		if(MorthalinRaidTools.db.global.itemCounter[player].item[string.sub(msg, 18, -2)] == nil) then
			MorthalinRaidTools.db.global.itemCounter[player].item[string.sub(msg, 18, -2)] = 1;
		else
			MorthalinRaidTools.db.global.itemCounter[player].item[string.sub(msg, 18, -2)] = MorthalinRaidTools.db.global.itemCounter[player].item[string.sub(msg, 18, -2)] + 1;
		end
	end
end

function MorthalinRaidTools:CheckVersion()
	if(GetAddOnMetadata("MorthalinRaidTools", "Version") ~= nil) then
		if(self.db.global.version ~= GetAddOnMetadata("MorthalinRaidTools", "Version")) then
			if(tonumber(self.db.global.version) < 1.31) then
				MorthalinRaidTools:ReconstructBuffs();
			end
			self:Print("Version of addon changed.");
			self.db.global.version = GetAddOnMetadata("MorthalinRaidTools", "Version");
		end
	end
end